#[cfg(feature = "Win32_Networking_ActiveDirectory")]
#[doc = "Required features: `\"Win32_Networking_ActiveDirectory\"`"]
pub mod ActiveDirectory;
#[cfg(feature = "Win32_Networking_Clustering")]
#[doc = "Required features: `\"Win32_Networking_Clustering\"`"]
pub mod Clustering;
#[cfg(feature = "Win32_Networking_HttpServer")]
#[doc = "Required features: `\"Win32_Networking_HttpServer\"`"]
pub mod HttpServer;
#[cfg(feature = "Win32_Networking_Ldap")]
#[doc = "Required features: `\"Win32_Networking_Ldap\"`"]
pub mod Ldap;
#[cfg(feature = "Win32_Networking_WebSocket")]
#[doc = "Required features: `\"Win32_Networking_WebSocket\"`"]
pub mod WebSocket;
#[cfg(feature = "Win32_Networking_WinHttp")]
#[doc = "Required features: `\"Win32_Networking_WinHttp\"`"]
pub mod WinHttp;
#[cfg(feature = "Win32_Networking_WinInet")]
#[doc = "Required features: `\"Win32_Networking_WinInet\"`"]
pub mod WinInet;
#[cfg(feature = "Win32_Networking_WinSock")]
#[doc = "Required features: `\"Win32_Networking_WinSock\"`"]
pub mod WinSock;
#[cfg(feature = "Win32_Networking_WindowsWebServices")]
#[doc = "Required features: `\"Win32_Networking_WindowsWebServices\"`"]
pub mod WindowsWebServices;
