#[cfg(feature = "Win32_Devices_AllJoyn")]
#[doc = "Required features: `\"Win32_Devices_AllJoyn\"`"]
pub mod AllJoyn;
#[cfg(feature = "Win32_Devices_BiometricFramework")]
#[doc = "Required features: `\"Win32_Devices_BiometricFramework\"`"]
pub mod BiometricFramework;
#[cfg(feature = "Win32_Devices_Bluetooth")]
#[doc = "Required features: `\"Win32_Devices_Bluetooth\"`"]
pub mod Bluetooth;
#[cfg(feature = "Win32_Devices_Communication")]
#[doc = "Required features: `\"Win32_Devices_Communication\"`"]
pub mod Communication;
#[cfg(feature = "Win32_Devices_DeviceAndDriverInstallation")]
#[doc = "Required features: `\"Win32_Devices_DeviceAndDriverInstallation\"`"]
pub mod DeviceAndDriverInstallation;
#[cfg(feature = "Win32_Devices_DeviceQuery")]
#[doc = "Required features: `\"Win32_Devices_DeviceQuery\"`"]
pub mod DeviceQuery;
#[cfg(feature = "Win32_Devices_Display")]
#[doc = "Required features: `\"Win32_Devices_Display\"`"]
pub mod Display;
#[cfg(feature = "Win32_Devices_Enumeration")]
#[doc = "Required features: `\"Win32_Devices_Enumeration\"`"]
pub mod Enumeration;
#[cfg(feature = "Win32_Devices_Fax")]
#[doc = "Required features: `\"Win32_Devices_Fax\"`"]
pub mod Fax;
#[cfg(feature = "Win32_Devices_HumanInterfaceDevice")]
#[doc = "Required features: `\"Win32_Devices_HumanInterfaceDevice\"`"]
pub mod HumanInterfaceDevice;
#[cfg(feature = "Win32_Devices_PortableDevices")]
#[doc = "Required features: `\"Win32_Devices_PortableDevices\"`"]
pub mod PortableDevices;
#[cfg(feature = "Win32_Devices_Properties")]
#[doc = "Required features: `\"Win32_Devices_Properties\"`"]
pub mod Properties;
#[cfg(feature = "Win32_Devices_Pwm")]
#[doc = "Required features: `\"Win32_Devices_Pwm\"`"]
pub mod Pwm;
#[cfg(feature = "Win32_Devices_Sensors")]
#[doc = "Required features: `\"Win32_Devices_Sensors\"`"]
pub mod Sensors;
#[cfg(feature = "Win32_Devices_SerialCommunication")]
#[doc = "Required features: `\"Win32_Devices_SerialCommunication\"`"]
pub mod SerialCommunication;
#[cfg(feature = "Win32_Devices_Tapi")]
#[doc = "Required features: `\"Win32_Devices_Tapi\"`"]
pub mod Tapi;
#[cfg(feature = "Win32_Devices_Usb")]
#[doc = "Required features: `\"Win32_Devices_Usb\"`"]
pub mod Usb;
#[cfg(feature = "Win32_Devices_WebServicesOnDevices")]
#[doc = "Required features: `\"Win32_Devices_WebServicesOnDevices\"`"]
pub mod WebServicesOnDevices;
