#[cfg(feature = "Wdk_Foundation")]
#[doc = "Required features: `\"Wdk_Foundation\"`"]
pub mod Foundation;
#[cfg(feature = "Wdk_Graphics")]
#[doc = "Required features: `\"Wdk_Graphics\"`"]
pub mod Graphics;
#[cfg(feature = "Wdk_Storage")]
#[doc = "Required features: `\"Wdk_Storage\"`"]
pub mod Storage;
#[cfg(feature = "Wdk_System")]
#[doc = "Required features: `\"Wdk_System\"`"]
pub mod System;
